@@ -5,12 +5,13 @@ import multiprocessing
 import os
 import pickle
 from itertools import product
-
+import matplotlib.pyplot as plt
 import numpy as np
+
 from joblib import Parallel, delayed, parallel_backend
 
-from manifoldy.definitions import DIMENSIONALITY_REDUCTION_MODELS
-from manifoldy.utils import setup_multiprocessing
+from definitions import DIMENSIONALITY_REDUCTION_MODELS
+from utils import setup_multiprocessing
 
 
 def apply_reduction(instance, model):
@@ -45,3 +46,12 @@ if __name__ == "__main__":
         with open("results/projection_dataset_names.pickle", "wb") as file:
             pickle.dump(projection_names, file)
         np.save("results/projection_results.npy", projection_data)
+
+
+i = 0
+plt.scatter(projection_data[i][:, 0], projection_data[i][:, 1])
+
+for i in range(10):
+    print(projection_names[i])
+    plt.scatter(projection_data[i][:, 0], projection_data[i][:, 1])
+    plt.show()
